package com.fossgalaxy.games.tbs.ai.rules;

import com.fossgalaxy.games.tbs.GameState;
import com.fossgalaxy.games.tbs.actions.AbstractAction;
import com.fossgalaxy.games.tbs.entity.Entity;
import com.fossgalaxy.object.annotations.ObjectDef;
import com.fossgalaxy.object.annotations.ObjectDefStatic;
import org.codetome.hexameter.core.api.CubeCoordinate;

import java.util.Collection;

public class AttackClosest extends FirstLegalRule {

    @ObjectDef("AttackClosest")
    public AttackClosest(){
        super(AbstractAction.CAT_ATTACK);
    }

    @ObjectDefStatic("AttackMelee")
    public static ProductionRule buildMelee() {
        return new AttackClosest();
    }

    @ObjectDefStatic("AttackRangedMostClosest")
    public static ProductionRule buildRanged() {
        return new AttackClosest();
    }

    @Override
    protected CubeCoordinate chooseTarget(Entity entity, GameState state) {
        CubeCoordinate ourPos = entity.getPos();

        Entity closestEnemy = null;
        double closestDist = Double.MAX_VALUE;

        Collection<Entity> entities = state.getEntities();
        for (Entity other : entities) {
            if (other.getOwner() == entity.getOwner()) {
                continue;
            }

            CubeCoordinate otherPos = other.getPos();
            double otherDist = state.getDistance(ourPos, otherPos);

            if (otherDist < closestDist) {
                closestEnemy = other;
                closestDist = otherDist;
            }

        }

        if (closestEnemy == null) {
            return null;
        }

        return closestEnemy.getPos();
    }
}
