package com.fossgalaxy.games.tbs.metrics;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.UUID;

import com.fossgalaxy.games.tbs.GameState;
import com.fossgalaxy.games.tbs.entity.Entity;
import com.fossgalaxy.games.tbs.order.Order;
import com.fossgalaxy.games.tbs.order.OrderProcessor;
import com.fossgalaxy.games.tbs.rules.Rule;

public class GameMetrics {
	private List<TurnMetrics> turns;
	
	private TurnMetrics currTurn;
	private Integer winner;

	private Rule winCondition;
	
	public GameMetrics() {
		this.turns = new ArrayList<>();
		this.winCondition = null;
	}
	
	public void updateActions(Map<UUID, Order> orders, GameState state) {
	}

	public void onTurnSetup(OrderProcessor orderProcessor, GameState state, int playerID) {
		if (currTurn != null) {
			throw new IllegalArgumentException("Metrics did not get logged before start of end turn.");
		}
		
		currTurn = new TurnMetrics(state.getTime(), playerID);
		currTurn.logStateStart(state);
	}

	/**
	 * An order has just been performed.
	 * 
	 * This gives the metrics the chance to process things.
	 * 
	 * @param entityID the ID of the entity
	 * @param order the order that the entity has made
	 * @param state the state after the move
	 */
	public void onOrder(UUID entityID, Order order, GameState state) {
		currTurn.logOrder(entityID, order);
	}

	/**
	 * An order has just been performed.
	 * 
	 * Unless you need to track automatic actions differently from manually triggered ones, you don't need to override this.
	 * 
	 * @param id the ID of the entity
	 * @param order the order that the entity has made
	 * @param state the state after the move
	 */
	public void onAutomaticOrder(UUID id, Order order, GameState state) {
		currTurn.logAutomatic(id);
	}

	/**
	 * Keep track of the entities which didn't have a manual or automatic order this turn.
	 * 
	 * @param entitiesNeedingOrders entities which didn't have orders.
	 */
	public void logIdleEntities(Set<Entity> entitiesNeedingOrders) {
		entitiesNeedingOrders.forEach(e -> currTurn.logIdle(e.getID()));
	}

	public void onTurnEnd(GameState state, Integer winner, Set<Integer> losers) {
		currTurn.logWinners(winner, losers);
		currTurn.logStateEnd(state);
		
		turns.add(currTurn);
		currTurn = null;
		
		if (winner != Rule.NO_WINNER) {
			this.winner = winner;
		}
	}

	public Integer getWinner() {
		return winner;
	}
	
	public int getTurns() {
		return turns.size();
	}
	
	public TurnMetrics getMetricsForTurn(int turnID) {
		return turns.get(turnID);
	}

    public void logWinCondition(Rule rule) {
		this.winCondition = rule;
    }

    public Rule getWinCondition(){
		return this.winCondition;
	}
}
